"""
Look in the autograder.cli package for modules that look like CLI tools
(has a _get_parser() method that returns an argparse parser)
and list them.
"""

import os
import sys

import autograder.code

THIS_DIR = os.path.abspath(os.path.dirname(os.path.realpath(__file__)))

def run():
    print("The autograder CLI package contains several tools for interacting with the autograder.")
    print("The following is a non-exhaustive list of CLI tools.")
    print("Invoke each command with the `--help` option for more details.")

    for dirent in sorted(os.listdir(THIS_DIR)):
        path = os.path.join(THIS_DIR, dirent)
        cmd = 'autograder.cli.' + os.path.splitext(dirent)[0]

        if (dirent.startswith('__')):
            continue

        try:
            module = autograder.code.import_path(path)
        except Exception:
            continue

        if ('_get_parser' not in dir(module)):
            continue

        parser = module._get_parser()
        parser.prog = 'python3 -m ' + cmd

        print()
        print(cmd)
        print(parser.description)
        parser.print_usage()

    return 0

def main():
    return run()

if (__name__ == '__main__'):
    sys.exit(main())



import unittest
import time

import autograder.question
import autograder.assignment

class TestAssignment(unittest.TestCase):
    class Q1(autograder.question.Question):
        def score_question(self, submission):
            result = submission()

            if (result):
                self.full_credit()
            else:
                self.fail("Got a False.")

    def test_base_full_credit(self):
        questions = [
            TestAssignment.Q1(1),
        ]

        class TA(autograder.assignment.Assignment):
            def _prepare_submission(self):
                return lambda: True

        assignment = TA('test_base_full_credit', questions)
        result = assignment.grade(show_exceptions = True)

        total_score, max_score = result.get_score()

        self.assertEqual(total_score, 1)
        self.assertEqual(max_score, 1)

    def test_base_fail(self):
        questions = [
            TestAssignment.Q1(1),
        ]

        class TA(autograder.assignment.Assignment):
            def _prepare_submission(self):
                return lambda: False

        assignment = TA('test_base_fail', questions)
        result = assignment.grade(show_exceptions = True)

        total_score, max_score = result.get_score()

        self.assertEqual(total_score, 0)
        self.assertEqual(max_score, 1)

    def test_sleep_fail(self):
        questions = [
            TestAssignment.Q1(1, timeout = 0.05),
        ]

        def submission():
            time.sleep(0.25)
            return True

        class TA(autograder.assignment.Assignment):
            def _prepare_submission(self):
                return submission

        # Shorten the reap time for testing.
        old_reap_time = autograder.utils.REAP_TIME_SEC
        autograder.utils.REAP_TIME_SEC = 0.01

        try:
            assignment = TA('test_sleep_fail', questions)
            result = assignment.grade(show_exceptions = True)
        finally:
            autograder.utils.REAP_TIME_SEC = old_reap_time

        total_score, max_score = result.get_score()

        self.assertEqual(total_score, 0)
        self.assertEqual(max_score, 1)



import sys

import autograder.api.common
import autograder.api.fetchgrades
import autograder.submission

HEADERS = ['user', 'has_submission', 'score', 'max_points', 'id', 'time']

def run(arguments):
    config_data = autograder.api.common.parse_config(arguments)
    success, result = autograder.api.fetchgrades.send(arguments.server, config_data)

    if (not success):
        print(result)
        return 1

    if (len(result) == 0):
        print("No submissions found.")
        return 0

    print("\t".join(HEADERS))

    for (email, entry) in result.items():
        row = []

        if (entry is None):
            row = [email, False, None, None, None, None]
        else:
            summary = autograder.submission.SubmissionSummary.from_dict(entry)
            row = [
                email, True,
                summary.score, summary.max_points,
                summary.short_id(), summary.pretty_time()
            ]

        print("\t".join([str(element) for element in row]))

    return 0

def _get_parser():
    parser = autograder.api.common.get_argument_parser(description =
            'Get all the most recent submissions for this assignment.')

    return parser

def main():
    return run(_get_parser().parse_args())

if (__name__ == '__main__'):
    sys.exit(main())



import argparse
import json
import os
import sys

import autograder.submission

DEFAULT_BASE_DIR = os.path.join('/', 'autograder')

DEFAULT_CONFIG_PATH = os.path.join(DEFAULT_BASE_DIR, 'config.json')

DEFAULT_INPUT_DIR = os.path.join(DEFAULT_BASE_DIR, 'input')
DEFAULT_OUTPUT_DIR = os.path.join(DEFAULT_BASE_DIR, 'output')
DEFAULT_WORK_DIR = os.path.join(DEFAULT_BASE_DIR, 'work')

def run(args):
    if (not os.path.exists(args.config)):
        # No config, there's nothing to do.
        return 0

    with open(args.config, 'r') as file:
        config = json.load(file)

    os.makedirs(args.basedir, exist_ok = True)
    os.makedirs(args.inputdir, exist_ok = True)
    os.makedirs(args.outputdir, exist_ok = True)
    os.makedirs(args.workdir, exist_ok = True)

    # Do post-submission file operations.
    # There are no pre-sub file ops and Docker has already copied over the submission files
    # into the input directory.
    for file_operation in config.get(autograder.submission.CONFIG_KEY_POST_SUB_OPS, []):
        autograder.submission.do_file_operation(file_operation, args.basedir)

    return 0

def _get_parser():
    parser = argparse.ArgumentParser(description =
        'Take any necessary steps before performing a standard docker-based grading.')

    parser.add_argument('-c', '--config',
        action = 'store', type = str, default = DEFAULT_CONFIG_PATH,
        help = 'The path to a JSON file describing grading configurations (default: %(default)s).')

    parser.add_argument('-bd', '--basedir',
        action = 'store', type = str, default = DEFAULT_BASE_DIR,
        help = 'The path to the base dir (where file operations are run) (default: %(default)s).')

    parser.add_argument('-id', '--inputdir',
        action = 'store', type = str, default = DEFAULT_INPUT_DIR,
        help = 'The path to the grading input directory (default: %(default)s).')

    parser.add_argument('-od', '--outputdir',
        action = 'store', type = str, default = DEFAULT_OUTPUT_DIR,
        help = 'The path to the grading output directory (default: %(default)s).')

    parser.add_argument('-wd', '--workdir',
        action = 'store', type = str, default = DEFAULT_WORK_DIR,
        help = 'The path to the grading work directory (default: %(default)s).')

    return parser

def main():
    return run(_get_parser().parse_args())

if (__name__ == '__main__'):
    sys.exit(main())



import argparse
import json
import os
import sys

import autograder.assignment
import autograder.submission
import autograder.utils

def run(args):
    grader_path = os.path.abspath(args.grader)
    work_dir = os.path.abspath(args.workdir)

    input_dir = args.inputdir
    if (input_dir is None):
        input_dir = work_dir
    input_dir = os.path.abspath(input_dir)

    output_dir = args.outputdir
    if (output_dir is None):
        output_dir = work_dir
    output_dir = os.path.abspath(output_dir)

    assignment_class = autograder.assignment.fetch_assignment(grader_path)

    result = autograder.submission.run_submission(assignment_class, input_dir, output_dir, work_dir)
    if (result is None):
        return 2

    print(result.report())

    if (args.outpath is not None):
        out_path = os.path.abspath(args.outpath)
        os.makedirs(os.path.dirname(os.path.abspath(out_path)), exist_ok = True)
        with open(out_path, 'w') as file:
            json.dump(result.to_dict(), file, indent = 4)

    return 0

def _get_parser():
    parser = argparse.ArgumentParser(description =
        'Grade a submission given an already prepared working (submission) directory'
        + ' and a grader file.'
        + ' Use autograder.cli.grade-assignment if you have not already prepared'
        + ' your work directory.')

    parser.add_argument('-g', '--grader',
        action = 'store', type = str, required = True,
        help = 'The path to a Python file containing a single assignment class.')

    parser.add_argument('-id', '--inputdir',
        action = 'store', type = str, required = False, default = None,
        help = 'The path to the submission\'s input directory (defaults to the work directory).')

    parser.add_argument('-od', '--outputdir',
        action = 'store', type = str, required = False, default = None,
        help = 'The path to the submission\'s output directory (defaults to the work directory).')

    parser.add_argument('-wd', '--workdir',
        action = 'store', type = str, required = True,
        help = 'The path to a submission to use for grading.')

    parser.add_argument('-o', '--outpath',
        action = 'store', type = str, required = False, default = None,
        help = 'The path to a output the JSON result.')

    parser.add_argument('-d', '--debug', dest = 'debug',
        action = 'store_true', default = False,
        help = 'Enable additional output and leave behind artifacts (default: %(default)s)')

    return parser

def main():
    return run(_get_parser().parse_args())

if (__name__ == '__main__'):
    sys.exit(main())



import sys

import autograder.cmd.gradeassignment

def main():
    return autograder.cmd.gradeassignment.main()

def _get_parser():
    return autograder.cmd.gradeassignment._get_parser()

if (__name__ == '__main__'):
    sys.exit(main())



import argparse
import json
import os
import sys

import autograder.submission

DEFAULT_ASSIGNMENT = 'assignment.json'

def run(args):
    assignment_path = os.path.abspath(args.assignment)
    submission_path = os.path.abspath(args.submission)

    dirs, assignment_class = autograder.submission.prep_temp_grading_dir(assignment_path,
        submission_path, debug = args.debug)

    input_dir, output_dir, work_dir = dirs

    result = autograder.submission.run_submission(assignment_class, input_dir, output_dir, work_dir)
    if (result is None):
        return 2

    print(result.report())

    if (args.outpath is not None):
        out_path = os.path.abspath(args.outpath)
        os.makedirs(os.path.dirname(os.path.abspath(out_path)), exist_ok = True)
        with open(out_path, 'w') as file:
            json.dump(result.to_dict(), file, indent = 4)

    return 0

def _get_parser():
    parser = argparse.ArgumentParser(description =
        'Grade an assignment (specified by an assignment JSON file) using the given submission.')

    parser.add_argument('-a', '--assignment',
        action = 'store', type = str, required = False, default = DEFAULT_ASSIGNMENT,
        help = 'The path to a JSON file describing an assignment (default: %(default)s).')

    parser.add_argument('-s', '--submission',
        action = 'store', type = str, required = True,
        help = 'The path to a submission to use for grading.')

    parser.add_argument('-o', '--outpath',
        action = 'store', type = str, required = False, default = None,
        help = 'The path to a output the JSON result.')

    parser.add_argument('-d', '--debug', dest = 'debug',
        action = 'store_true', default = False,
        help = 'Enable additional output and leave behind artifacts (default: %(default)s).')

    return parser

def main():
    return run(_get_parser().parse_args())

if (__name__ == '__main__'):
    sys.exit(main())



import os
import sys
import traceback

import autograder.api.common
import autograder.api.submit
import autograder.submission

def _get_files(test_submission):
    paths = []

    test_submission = os.path.abspath(test_submission)

    submission_dir = os.path.dirname(test_submission)
    for dirent in os.listdir(submission_dir):
        path = os.path.join(submission_dir, dirent)
        if (not os.path.samefile(test_submission, path)):
            paths.append(path)

    return paths

def run(arguments):
    config_data = autograder.api.common.parse_config(arguments)

    try:
        test_submissions = autograder.submission.fetch_test_submissions(arguments.submissions)
    except Exception as ex:
        print("Failed to load submission(s) from '%s': '%s'." % (arguments.submissions, ex))
        traceback.print_exc()
        return 101

    errors = 0
    for test_submission in test_submissions:
        paths = _get_files(test_submission)

        try:
            success, result = autograder.api.submit.send(arguments.server, config_data, paths)
        except Exception as ex:
            print("Failed to run submission '%s': '%s'." % (test_submission, ex))
            traceback.print_exc()

        if (not success):
            print("Autograder failed to grade the submission: " + result)
            errors += 1
            continue

        if (not autograder.submission.compare_test_submission(test_submission, result)):
            errors += 1

    print("Encountered %d error(s) while testing %d submissions." % (errors, len(test_submissions)))

    if (errors > 0):
        print("Faiure")
    else:
        print("Success")

    return errors

def _get_parser():
    parser = autograder.api.common.get_argument_parser(description =
        'Submit multiple assignments to an autograder and ensure the output is expected.')

    parser.add_argument('-s', '--submissions',
        action = 'store', type = str, required = True,
        help = 'The path to a dir containing one or more test submissions.')

    return parser

def main():
    return run(_get_parser().parse_args())

if (__name__ == '__main__'):
    sys.exit(main())



import glob
import json
import os
import shutil
import traceback

import autograder.assignment
import autograder.git
import autograder.utils

TEST_SUBMISSION_FILENAME = 'test-submission.json'
GRADER_FILENAME = 'grader.py'

CONFIG_KEY_STATIC_FILES = 'static-files'
CONFIG_KEY_PRE_STATIC_OPS = 'pre-static-files-ops'
CONFIG_KEY_POST_STATIC_OPS = 'post-static-files-ops'
CONFIG_KEY_POST_SUB_OPS = 'post-submission-files-ops'

FILESPEC_DELIM = '::'
FILESPEC_GIT = 'git'
FILESPEC_GIT_PREFIX = FILESPEC_GIT + FILESPEC_DELIM

def do_file_operation(operation, op_dir):
    if ((operation is None) or (len(operation) == 0)):
        raise ValueError("File operation is empty.")

    if (operation[0] == 'mv'):
        if (len(operation) != 3):
            raise ValueError("Incorrect number of argument for 'mv' file operation."
                + " Expected 2, found %d." % ((len(operation) - 1)))

        shutil.move(os.path.join(op_dir, operation[1]), os.path.join(op_dir, operation[2]))
    elif (operation[0] == 'cp'):
        if (len(operation) != 3):
            raise ValueError("Incorrect number of argument for 'cp' file operation."
                + " Expected 2, found %d." % ((len(operation) - 1)))

        autograder.utils.copy_dirent(os.path.join(op_dir, operation[1]), os.path.join(op_dir,
            operation[2]))
    else:
        raise ValueError("Unknown file operation: '%s'." % (operation[0]))

def handle_git_static_file(filespec, dest_dir):
    parts = filespec.split(FILESPEC_DELIM)

    if ((len(parts) < 2) or (len(parts) > 4) or (parts[0] != FILESPEC_GIT)):
        raise ValueError("Unknown git filespec: '%s'." % (filespec))

    url = parts[1]

    dirname = url.split('/')[-1].removesuffix('.git')
    if (len(parts) >= 3):
        dirname = parts[2]

    ref = None
    if (len(parts) >= 4):
        ref = parts[3]

    dest_path = os.path.join(dest_dir, dirname)

    autograder.git.ensure_repo(url, dest_path, update = True, ref = ref)

def copy_assignment_file(filename, source_dir, dest_dir, only_contents):
    source_path = os.path.join(source_dir, filename)
    dest_path = os.path.join(dest_dir, os.path.basename(filename))

    if (only_contents):
        autograder.utils.copy_dirent_contents(source_path, dest_path)
    else:
        autograder.utils.copy_dirent(source_path, dest_path)

def copy_assignment_files(source_dir, dest_dir, op_dir, files,
        only_contents = False, pre_ops = [], post_ops = []):
    """
    Copy over assignment files:
    1) Do pre-copy operations.
    2) Copy.
    3) Do post-copy operations.
    """

    # Do pre operations.
    for file_operation in pre_ops:
        do_file_operation(file_operation, op_dir)

    # Copy over the assignment's files.
    for filespec in files:
        if (filespec.startswith(FILESPEC_GIT_PREFIX)):
            handle_git_static_file(filespec, dest_dir)
        else:
            copy_assignment_file(filespec, source_dir, dest_dir, only_contents)

    # Do post operations.
    for file_operation in post_ops:
        do_file_operation(file_operation, op_dir)

def fetch_test_submissions(path):
    path = os.path.abspath(path)
    test_submissions = []

    if (os.path.isfile(path)):
        if (os.path.basename(path) != TEST_SUBMISSION_FILENAME):
            raise ValueError("Passed in submission is not named like a test submission ('%s')." % (
                TEST_SUBMISSION_FILENAME))

        test_submissions.append(path)
    else:
        test_submissions += glob.glob(os.path.join(path, '**', TEST_SUBMISSION_FILENAME),
                recursive = True)

    return test_submissions

def prep_temp_grading_dir(assignment_config_path, submission_dir, debug = False):
    """
    Create a directory for grading a submission.
    1) Create a temp dir.
    2) Create the three core directories (input/output/work) in the temp dir.
    3) Copy over the static files (includng pre/post operations).
    4) Copy over the submission files (includng pre/post operations).
    5) Load the grader class.
    6) Return the temp dir and grader class.
    """

    temp_dir = autograder.utils.get_temp_path(prefix = 'autograder-submission-',
            rm = (not debug))
    os.makedirs(temp_dir)

    if (debug):
        print("Using temp/work dir: '%s'." % (temp_dir))

    input_dir, output_dir, work_dir = prep_grading_dir(assignment_config_path, temp_dir,
        submission_dir)

    grader_path = os.path.join(work_dir, GRADER_FILENAME)
    assignment_class = autograder.assignment.fetch_assignment(grader_path)

    return ((input_dir, output_dir, work_dir), assignment_class)

def make_core_dirs(base_dir):
    """
    Make the three core directories.
    """

    input_dir = os.path.join(base_dir, 'input')
    os.makedirs(input_dir, exist_ok = True)

    output_dir = os.path.join(base_dir, 'output')
    os.makedirs(output_dir, exist_ok = True)

    work_dir = os.path.join(base_dir, 'work')
    os.makedirs(work_dir, exist_ok = True)

    return input_dir, output_dir, work_dir

def prep_grading_dir(assignment_config_path, base_dir, submission_dir, skip_static = False):
    input_dir, output_dir, work_dir = make_core_dirs(base_dir)

    # Load the assignment config.

    assignment_config_path = os.path.abspath(assignment_config_path)
    assignment_base_dir = os.path.dirname(assignment_config_path)

    try:
        with open(assignment_config_path, 'r') as file:
            assignment_config = json.load(file)
    except Exception as ex:
        raise ValueError("Failed to load assignment config: " + assignment_config_path) from ex

    if (not skip_static):
        # Copy static files.
        copy_assignment_files(assignment_base_dir, work_dir, base_dir,
                assignment_config.get(CONFIG_KEY_STATIC_FILES, []),
                pre_ops = assignment_config.get(CONFIG_KEY_PRE_STATIC_OPS, []),
                post_ops = assignment_config.get(CONFIG_KEY_POST_STATIC_OPS, []))

    # Copy submission files.
    copy_assignment_files(submission_dir, input_dir, base_dir,
        ['.'], only_contents = True,
        pre_ops = [],
        post_ops = assignment_config.get(CONFIG_KEY_POST_SUB_OPS, []))

    return input_dir, output_dir, work_dir

def run_test_submission(assignment_config_path, submission_config_path, debug = False):
    print("Testing assignment '%s' and submission '%s'." % (assignment_config_path,
        submission_config_path))

    dirs, assignment_class = prep_temp_grading_dir(assignment_config_path,
        os.path.dirname(submission_config_path), debug = debug)
    input_dir, output_dir, work_dir = dirs

    if (assignment_class is None):
        return False

    actual_result = run_submission(assignment_class, input_dir, output_dir, work_dir)
    if (actual_result is None):
        return False

    return compare_test_submission(submission_config_path, actual_result)

def compare_test_submission(test_config_path, actual_result, print_result = True):
    with open(test_config_path, 'r') as file:
        test_config = json.load(file)

    expected_result = autograder.assignment.GradedAssignment.from_dict(test_config['result'])
    ignore_messages = test_config.get('ignore_messages', False)

    match = actual_result.equals(expected_result, ignore_messages = ignore_messages)

    if ((not match) and print_result):
        print("Submission does not match expected output: '%s'." % (test_config_path))
        print('Expected:')
        print(expected_result.report(prefix = '    '))
        print('---')
        print('Actual:')
        print(actual_result.report(prefix = '    '))
        print('---')

    return match

def run_submission(assignment_class, input_dir, output_dir, work_dir):
    try:
        assignment = assignment_class(input_dir = input_dir, output_dir = output_dir,
                work_dir = work_dir)
        return assignment.grade()
    except Exception as ex:
        print("Failed to run assignment (%s) on submission '%s': '%s'." % (
            assignment_class.__name__, input_dir, ex))
        traceback.print_exc()
        return None

class SubmissionSummary(object):
    """
    A summary of a grading submission.
    """

    def __init__(self,
            id = '',
            max_points = 0, score = 0,
            message = '',
            grading_start_time = None,
            **kwargs):
        self.id = id

        self.max_points = max_points
        self.score = score

        self.message = message

        self.grading_start_time = None
        if (grading_start_time is not None):
            self.grading_start_time = autograder.utils.get_timestamp(grading_start_time)

    def to_dict(self):
        """
        Convert to all simple structures that can be later converted to JSON.
        """

        return {
            'id': self.id,
            'max_points': self.max_points,
            'score': self.score,
            'message': self.message,
            'grading_start_time': autograder.utils.timestamp_to_string(self.grading_start_time),
        }

    @staticmethod
    def from_dict(data):
        """
        Partner to to_dict().
        """

        return SubmissionSummary(**data)

    def short_id(self):
        return self.id.split('::')[-1]

    def pretty_time(self):
        return autograder.utils.timestamp_to_string(self.grading_start_time, pretty = True)

    def __repr__(self):
        """
        Get a string that represents the summary.
        """

        message = '.'
        if ((self.message is not None) and (self.message != '')):
            message = ", Message: '%s'." % (self.message)

        return "Submission ID: %s, Score: %s / %s, Time: %s%s" % (
            self.short_id(), self.score, self.max_points,
            self.pretty_time(),
            message)



import autograder.api.common
import autograder.assignment

API_ENDPOINT = '/api/v01/submit'
API_KEYS = ['user', 'pass', 'course', 'assignment', 'message']

def send(server, config_data, paths):
    """
    Take in a server address
    and config data (of the form produced by autograder.api.common.parse_config()),
    and make a subimt request.
    Returns:
        (success, <message or graded assignment>)
    """

    url = "%s%s" % (server, API_ENDPOINT)

    data = {}
    for key in API_KEYS:
        if (config_data.get(key) is None):
            return (False, "No request made, missing required config '%s'." % (key))

        data[key] = config_data[key]

    body, message = autograder.api.common.send_api_request(url, data = data, files = paths)

    if (body is None):
        response = "The autograder failed to grade your assignment."
        response += "\nMessage from the autograder: " + message
        return (False, response)

    result = autograder.assignment.GradedAssignment.from_dict(body['result'])

    return (True, result)



import sys

import autograder.api.common
import autograder.api.history
import autograder.submission

def run(arguments):
    config_data = autograder.api.common.parse_config(arguments)
    success, result = autograder.api.history.send(arguments.server, config_data)

    if (not success):
        print(result)
        return 1

    if (len(result) == 0):
        print("No past submission found for this assignment.")
        return 0

    print("Found %d submissions." % (len(result)))
    for entry in result:
        print("    " + str(autograder.submission.SubmissionSummary.from_dict(entry)))

    return 0

def _get_parser():
    parser = autograder.api.common.get_argument_parser(description =
            'Get a history of past submission for this assignment.')

    return parser

def main():
    return run(_get_parser().parse_args())

if (__name__ == '__main__'):
    sys.exit(main())



import sys

import autograder.api.common
import autograder.api.submit

def run(arguments):
    config_data = autograder.api.common.parse_config(arguments)
    config_data['message'] = arguments.message

    success, result = autograder.api.submit.send(arguments.server, config_data, arguments.files)

    if (not success):
        print(result)
        return 1

    print('The autograder successfully graded your assignment.')
    print(result.report())

    return 0

def _get_parser():
    parser = autograder.api.common.get_argument_parser(description = 'Submit an assignment.')

    parser.add_argument('--message', dest = 'message',
        action = 'store', type = str, default = '',
        help = 'message')

    parser.add_argument('files', metavar = 'FILE',
        action = 'store', type = str, nargs = '+',
        help = 'The path to your submission file.')

    return parser

def main():
    return run(_get_parser().parse_args())

if (__name__ == '__main__'):
    sys.exit(main())



import argparse
import json
import os

import requests

import autograder.util.hash

DEFAULT_CONFIG_PATH = 'config.json'
DEFAULT_AUTOGRADER_URL = 'http://lighthouse.soe.ucsc.edu'

API_REQUEST_JSON_KEY = 'content'

API_RESPONSE_KEY_SUCCESS = 'success'
API_RESPONSE_KEY_CONTENT = 'content'

def send_api_request(url, method = None, data = {}, files = [], clean = True):
    """
    Make an API request and respond with:
     - on success -- (the response body, None)
     - on failure -- (None, a failure message)
    """

    post_files = {}
    for path in files:
        filename = os.path.basename(path)
        if (filename in post_files):
            raise ValueError("Cannot submit duplicate filenames ('%s')." % (filename))

        post_files[filename] = open(path, 'rb')

    if (method is None):
        if ((len(files) > 0) or (len(data) > 0)):
            method = 'POST'
        else:
            method = 'GET'

    if (clean):
        # Don't send cleartext passwords.
        if ('pass' in data):
            data['pass'] = autograder.util.hash.sha256_hex(data['pass'])

    raw_response = requests.request(
        method = method,
        url = url,
        data = {API_REQUEST_JSON_KEY: json.dumps(data)},
        files = post_files)

    for file in post_files.values():
        file.close()

    if (raw_response.status_code == 401):
        return None, ("Request could not be authenticated."
            + " Ensure that your username, password, and course are properly set.")

    if (raw_response.status_code == 403):
        return None, "You do not have sufficient permissions for that operation."

    if (raw_response.status_code != 200):
        return None, "Recieved a failure status from the autograding server: %d." % (
            raw_response.status_code)

    try:
        response = raw_response.json()
    except Exception:
        return None, "Autograder response does not contain valid JSON. Response:\n---\n%s\n---" % (
            raw_response.text)

    if (not response.get(API_RESPONSE_KEY_SUCCESS, False)):
        message = response.get(API_RESPONSE_KEY_CONTENT, "Request to autograding server failed.")
        return None, message

    return response.get(API_RESPONSE_KEY_CONTENT, None), None

def parse_config(arguments):
    """
    Given the standard CLI options (from get_argument_parser()),
    parse out the full API configuration.
    """

    config = {
        'user': None,
        'pass': None,
        'course': None,
        'assignment': None,
    }

    if ((arguments.config_paths is not None) and (len(arguments.config_paths) > 0)):
        for path in arguments.config_paths:
            with open(path, 'r') as file:
                config.update(json.load(file))
    elif (os.path.isfile(DEFAULT_CONFIG_PATH)):
        with open(DEFAULT_CONFIG_PATH, 'r') as file:
            config.update(json.load(file))

    for (key, value) in vars(arguments).items():
        if (key == 'config_paths'):
            continue

        if ((value is None) or (value == '')):
            continue

        config[key] = value

    return config

def get_argument_parser(
        description = 'Send an API request to the autograder.'):
    """
    Create an argparse parser that has all the standard options for API requests.
    """

    parser = argparse.ArgumentParser(description = description)

    parser.add_argument('--config', dest = 'config_paths',
        action = 'append', type = str,
        help = 'A JSON config file with your submission/authentication details.'
            + " Can be specified multiple times with later values overriding earlier ones."
            + " If not specified, '%s' will be checked." % (DEFAULT_CONFIG_PATH)
            + ' These options can be set directly using other CLI arguments.')

    parser.add_argument('--user', dest = 'user',
        action = 'store', type = str, default = None,
        help = 'username')

    parser.add_argument('--pass', dest = 'pass',
        action = 'store', type = str, default = None,
        help = 'password')

    parser.add_argument('--course', dest = 'course',
        action = 'store', type = str, default = None,
        help = 'course')

    parser.add_argument('--assignment', dest = 'assignment',
        action = 'store', type = str, default = None,
        help = 'assignment')

    parser.add_argument('--server', dest = 'server',
        action = 'store', type = str, default = DEFAULT_AUTOGRADER_URL,
        help = 'The URL of the server to submit to (default: %(default)s).')

    return parser
